# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-05 17:41
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_users', '0006_auto_20161005_1651'),
    ]

    operations = [
        migrations.RenameField(
            model_name='passwordrecoverytoken',
            old_name='used',
            new_name='used_date',
        ),
    ]
